@@ -80,16 +80,21 @@ static int labelToEnumValue(const QCString &l)
   return (it!=s_labels.end()) ? it->second : 0;
 }
 
-int Debug::setFlag(const QCString &lab)
+int Debug::setFlagStr(const QCString &lab)
 {
   int retVal = labelToEnumValue(lab);
   curMask = static_cast<DebugMask>(curMask | retVal);
   return retVal;
 }
 
-void Debug::clearFlag(const QCString &lab)
+void Debug::setFlag(const DebugMask mask)
 {
-  curMask = static_cast<DebugMask>(curMask & ~labelToEnumValue(lab));
+  curMask = static_cast<DebugMask>(curMask | mask);
+}
+
+void Debug::clearFlag(const DebugMask mask)
+{
+  curMask = static_cast<DebugMask>(curMask & ~mask);
 }
 
 void Debug::setPriority(int p)
@@ -117,16 +122,17 @@ class Timer
   public:
     void start()
     {
-      m_startTime = std::chrono::system_clock::now();
+      m_startTime = std::chrono::steady_clock::now();
     }
     double elapsedTimeS()
     {
-      return (std::chrono::duration_cast<
+      return static_cast<double>(
+              std::chrono::duration_cast<
                   std::chrono::microseconds>(
-                  std::chrono::system_clock::now() - m_startTime).count()) / 1000000.0;
+                  std::chrono::steady_clock::now() - m_startTime).count()) / 1000000.0;
     }
   private:
-    std::chrono::time_point<std::chrono::system_clock> m_startTime;
+    std::chrono::time_point<std::chrono::steady_clock> m_startTime;
 };
 
 static Timer g_runningTime;
