@@ -12,7 +12,6 @@
  */
 
 #include <algorithm>
-#include <fstream>
 #include <memory>
 #include <string.h>
 #include <vector>
@@ -27,6 +26,7 @@
 #include "qhp.h"
 #include "textstream.h"
 #include "util.h"
+#include "portable.h"
 
 static inline void writeIndent(TextStream &t,int indent)
 {
@@ -207,7 +207,7 @@ void Qhp::initialize()
   ..
   */
   QCString fileName = Config_getString(HTML_OUTPUT) + "/" + qhpFileName;
-  p->docFile.open( fileName.str(), std::ofstream::out | std::ofstream::binary);
+  p->docFile = Portable::openOutputStream(fileName);
   if (!p->docFile.is_open())
   {
     term("Could not open file %s for writing\n", fileName.data());
