@@ -2,7 +2,7 @@
  *
  *
  *
- * Copyright (C) 1997-2013 by Dimitri van Heesch.
+ * Copyright (C) 1997-2015 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby
@@ -16,7 +16,7 @@
  */
 // Тranslated by Slave Jovanovski <slavejovanovski@yahoo.com>
 //
-// The cyrilic strings were entered using Macedonian language support in
+// The cyrillic strings were entered using Macedonian language support in
 // Windows. The editor used was Eclipse 3.2. The file was saved in UTF-8.
 //
 // Updates:
@@ -65,6 +65,14 @@ class TranslatorMacedonian : public TranslatorAdapter_1_6_0
     {
       return "\\usepackage[macedonian]{babel}\n";
     }
+    virtual QCString trISOLang()
+    {
+      return "mk";
+    }
+    virtual QCString getLanguageString()
+    {
+      return "0x042f Macedonian (Former Yugoslav Republic of Macedonia)";
+    }
 
     // --- Language translation methods -------------------
 
@@ -80,6 +88,10 @@ class TranslatorMacedonian : public TranslatorAdapter_1_6_0
     virtual QCString trDetailedDescription()
     { return "Детален опис"; }
 
+    /*! header that is used when the summary tag is missing inside the details tag */
+    virtual QCString trDetails()
+    { return "Детали"; }
+
     /*! header that is put before the list of typedefs. */
     virtual QCString trMemberTypedefDocumentation()
     { return "Документација на членови дефиниции на тип"; }
@@ -95,7 +107,7 @@ class TranslatorMacedonian : public TranslatorAdapter_1_6_0
     /*! header that is put before the list of member attributes. */
     virtual QCString trMemberDataDocumentation()
     {
-      //if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+      //if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
       {
         return "Документација на членови";
       }
@@ -124,9 +136,9 @@ class TranslatorMacedonian : public TranslatorAdapter_1_6_0
     /*! this is put at the author sections at the bottom of man pages.
      *  parameter s is name of the project name.
      */
-    virtual QCString trGeneratedAutomatically(const char *s)
+    virtual QCString trGeneratedAutomatically(const QCString &s)
     { QCString result="Автоматски создадено од Doxygen";
-      if (s) result+=(QCString)" за "+s;
+      if (!s.isEmpty()) result+=" за "+s;
       result+=" изворниот код.";
       return result;
     }
@@ -158,7 +170,7 @@ class TranslatorMacedonian : public TranslatorAdapter_1_6_0
     /*! This is put above each page as a link to the list of annotated classes */
     virtual QCString trCompoundList()
     {
-      if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+      if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
       {
         return "Список на Структури";
       }
@@ -175,7 +187,7 @@ class TranslatorMacedonian : public TranslatorAdapter_1_6_0
     /*! This is put above each page as a link to all members of compounds. */
     virtual QCString trCompoundMembers()
     {
-      if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+      if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
       {
         return "Членови";
       }
@@ -188,7 +200,7 @@ class TranslatorMacedonian : public TranslatorAdapter_1_6_0
     /*! This is put above each page as a link to all members of files. */
     virtual QCString trFileMembers()
     {
-      if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+      if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
       {
         return "Глобални Членови";
       }
@@ -228,10 +240,14 @@ class TranslatorMacedonian : public TranslatorAdapter_1_6_0
     virtual QCString trCompoundListDescription()
     {
 
-      if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+      if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
       {
         return "Список на структури со кратки описи:";
       }
+      else if (Config_getBool(OPTIMIZE_OUTPUT_SLICE))
+      {
+        return "Список на класи со кратки описи:";
+      }
       else
       {
         return "Список на класи, структури, унии и интерфејси "
@@ -247,7 +263,7 @@ class TranslatorMacedonian : public TranslatorAdapter_1_6_0
       {
         result+="документирани ";
       }
-      if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+      if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
       {
         result+="структури и унии";
       }
@@ -258,7 +274,7 @@ class TranslatorMacedonian : public TranslatorAdapter_1_6_0
       result+=" со врски до ";
       if (!extractAll)
       {
-        if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+        if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
         {
           result+="документацијата за секој член на структурата/унијата:";
         }
@@ -269,7 +285,7 @@ class TranslatorMacedonian : public TranslatorAdapter_1_6_0
       }
       else
       {
-        if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+        if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
         {
           result+="структурите/униите на кои што припаѓаат:";
         }
@@ -287,7 +303,7 @@ class TranslatorMacedonian : public TranslatorAdapter_1_6_0
       QCString result="Список на сите ";
       if (!extractAll) result+="документирани ";
 
-      if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+      if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
       {
         result+="функции, променливи, дефиниции, набројувања и дефиниции на тип";
       }
@@ -338,7 +354,7 @@ class TranslatorMacedonian : public TranslatorAdapter_1_6_0
      */
     virtual QCString trCompoundIndex()
     {
-      if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+      if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
       {
         return "Попис на Структури";
       }
@@ -365,10 +381,14 @@ class TranslatorMacedonian : public TranslatorAdapter_1_6_0
      */
     virtual QCString trClassDocumentation()
     {
-      if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+      if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
       {
         return "Документација на Структури";
       }
+      else if (Config_getBool(OPTIMIZE_OUTPUT_VHDL))
+      {
+          return trDesignUnitDocumentation();
+      }
       else
       {
         return "Документација на Класи";
@@ -387,12 +407,6 @@ class TranslatorMacedonian : public TranslatorAdapter_1_6_0
     virtual QCString trExampleDocumentation()
     { return "Документаија на Примери"; }
 
-    /*! This is used in LaTeX as the title of the chapter containing
-     *  the documentation of all related pages.
-     */
-    virtual QCString trPageDocumentation()
-    { return "Документација на Страници"; }
-
     /*! This is used in LaTeX as the title of the document */
     virtual QCString trReferenceManual()
     { return "Прирачник"; }
@@ -468,7 +482,7 @@ class TranslatorMacedonian : public TranslatorAdapter_1_6_0
      */
     virtual QCString trCompounds()
     {
-      if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+      if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
       {
         return "Структури";
       }
@@ -481,24 +495,20 @@ class TranslatorMacedonian : public TranslatorAdapter_1_6_0
     /*! This is used in the standard footer of each page and indicates when
      *  the page was generated
      */
-    virtual QCString trGeneratedAt(const char *date,const char *projName)
+    virtual QCString trGeneratedAt(const QCString &date,const QCString &projName)
     {
-      QCString result=(QCString)"Создадено на "+date;
-      if (projName) result+=(QCString)" за "+projName;
-      result+=(QCString)" од";
+      QCString result="Создадено на "+date;
+      if (!projName.isEmpty()) result+=" за "+projName;
+      result+=" од";
       return result;
     }
 
     /*! this text is put before a class diagram */
-    virtual QCString trClassDiagram(const char *clName)
+    virtual QCString trClassDiagram(const QCString &clName)
     {
-      return (QCString)"Диаграм на наследување за "+clName+":";
+      return "Диаграм на наследување за "+clName+":";
     }
 
-    /*! this text is generated when the \\internal command is used. */
-    virtual QCString trForInternalUseOnly()
-    { return "Само за интерна употреба."; }
-
     /*! this text is generated when the \\warning command is used. */
     virtual QCString trWarning()
     { return "Предупредување"; }
@@ -569,11 +579,11 @@ class TranslatorMacedonian : public TranslatorAdapter_1_6_0
 //////////////////////////////////////////////////////////////////////////
 
     /*! used as the title of the HTML page of a class/struct/union */
-    virtual QCString trCompoundReference(const char *clName,
+    virtual QCString trCompoundReference(const QCString &clName,
                                     ClassDef::CompoundType compType,
                                     bool isTemplate)
     {
-      QCString result=(QCString)clName;
+      QCString result=clName;
       switch(compType)
       {
         case ClassDef::Class:      result+=" Класа"; break;
@@ -591,7 +601,7 @@ class TranslatorMacedonian : public TranslatorAdapter_1_6_0
     }
 
     /*! used as the title of the HTML page of a file */
-    virtual QCString trFileReference(const char *fileName)
+    virtual QCString trFileReference(const QCString &fileName)
     {
       QCString result = "Опис на Датотекaта ";
       result += fileName;
@@ -599,7 +609,7 @@ class TranslatorMacedonian : public TranslatorAdapter_1_6_0
     }
 
     /*! used as the title of the HTML page of a namespace */
-    virtual QCString trNamespaceReference(const char *namespaceName)
+    virtual QCString trNamespaceReference(const QCString &namespaceName)
     {
       QCString result = "Опис на Имeто на Простор ";
       result += namespaceName;
@@ -733,7 +743,7 @@ class TranslatorMacedonian : public TranslatorAdapter_1_6_0
         bool single)
     { // here s is one of " Class", " Struct" or " Union"
       // single is true implies a single file
-      QCString result=(QCString)"Документацијата за ";
+      QCString result="Документацијата за ";
       switch(compType)
       {
         case ClassDef::Class:      result+="оваа класа"; break;
@@ -796,14 +806,14 @@ class TranslatorMacedonian : public TranslatorAdapter_1_6_0
 //////////////////////////////////////////////////////////////////////////
 
     /*! this text is put before a collaboration diagram */
-    virtual QCString trCollaborationDiagram(const char *clName)
+    virtual QCString trCollaborationDiagram(const QCString &clName)
     {
-      return (QCString)"Диаграм на соработка за "+clName+":";
+      return "Диаграм на соработка за "+clName+":";
     }
     /*! this text is put before an include dependency graph */
-    virtual QCString trInclDepGraph(const char *fName)
+    virtual QCString trInclDepGraph(const QCString &fName)
     {
-      return (QCString)"Вклучен дијаграм на зависност за "+fName+":";
+      return "Вклучен дијаграм на зависност за "+fName+":";
     }
     /*! header that is put before the list of constructor/destructors. */
     virtual QCString trConstructorDocumentation()
@@ -876,7 +886,7 @@ class TranslatorMacedonian : public TranslatorAdapter_1_6_0
     }
     virtual QCString trPublicAttribs()
     {
-      if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+      if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
       {
         return "Членови";
       }
@@ -1003,7 +1013,7 @@ class TranslatorMacedonian : public TranslatorAdapter_1_6_0
         "\\endcode\n"
         "Ако вредноста на \\c MAX_DOT_GRAPH_HEIGHT во конфигурациската датотека "
         "е 240 тогаш примерот ќе го создаде следниов дијаграм:"
-        "<p><center><img alt=\"\" src=\"graph_legend."+Config_getEnum("DOT_IMAGE_FORMAT")+"\"></center>\n"
+        "<p><center><img alt=\"\" src=\"graph_legend."+getDotImageExtension()+"\"></center>\n"
         "<p>\n"
         "Правоаголниците во дијаграмот погоре го имаат следново значење:\n"
         "<ul>\n"
@@ -1071,7 +1081,7 @@ class TranslatorMacedonian : public TranslatorAdapter_1_6_0
     /*! Used for Java classes in the summary section of Java packages */
     virtual QCString trClasses()
     {
-      if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+      if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
       {
         return "Структури";
       }
@@ -1081,14 +1091,9 @@ class TranslatorMacedonian : public TranslatorAdapter_1_6_0
       }
     }
     /*! Used as the title of a Java package */
-    virtual QCString trPackage(const char *name)
+    virtual QCString trPackage(const QCString &name)
     {
-      return (QCString)"Пакет "+name;
-    }
-    /*! Title of the package index page */
-    virtual QCString trPackageList()
-    {
-      return "Список на Пакети";
+      return "Пакет "+name;
     }
     /*! The description of the package index page */
     virtual QCString trPackageListDescription()
@@ -1345,14 +1350,18 @@ class TranslatorMacedonian : public TranslatorAdapter_1_6_0
     /*! Used as a heading for a list of Java class functions with package
      * scope.
      */
-    virtual QCString trPackageMembers()
+    virtual QCString trPackageFunctions()
     {
       return "Функции во Пакетот";
     }
+    virtual QCString trPackageMembers()
+    {
+      return "Членови во Пакетот";
+    }
     /*! Used as a heading for a list of static Java class functions with
      *  package scope.
      */
-    virtual QCString trStaticPackageMembers()
+    virtual QCString trStaticPackageFunctions()
     {
       return "Статични Функции во Пакетот";
     }
@@ -1464,16 +1473,10 @@ class TranslatorMacedonian : public TranslatorAdapter_1_6_0
     virtual QCString trDirectories()
     { return "Именици"; }
 
-    /*! This returns a sentences that introduces the directory hierarchy.
-     *  and the fact that it is sorted alphabetically per level
-     */
-    virtual QCString trDirDescription()
-    { return "Ова стебло на именици е приближно азбучно подреден:";}
-
     /*! This returns the title of a directory page. The name of the
      *  directory is passed via \a dirName.
      */
-    virtual QCString trDirReference(const char *dirName)
+    virtual QCString trDirReference(const QCString &dirName)
     { QCString result= "Опис на Именикот "; result += dirName; return result; }
 
     /*! This returns the word directory with or without starting capital
@@ -1600,11 +1603,11 @@ class TranslatorMacedonian : public TranslatorAdapter_1_6_0
     }
 
     /*! used as the title of the HTML page of a module/type (Fortran) */
-    virtual QCString trCompoundReferenceFortran(const char *clName,
+    virtual QCString trCompoundReferenceFortran(const QCString &clName,
                                     ClassDef::CompoundType compType,
                                     bool isTemplate)
     {
-      QCString result=(QCString)clName + " - Повикување на";
+      QCString result=clName + " - Повикување на";
       switch(compType)
       {
         case ClassDef::Class:      result+=" Класа"; break;
@@ -1620,7 +1623,7 @@ class TranslatorMacedonian : public TranslatorAdapter_1_6_0
       return result;
     }
     /*! used as the title of the HTML page of a module (Fortran) */
-    virtual QCString trModuleReference(const char *namespaceName)
+    virtual QCString trModuleReference(const QCString &namespaceName)
     {
       QCString result=namespaceName;
       result+=" - Содржина на Модул";
@@ -1671,7 +1674,7 @@ class TranslatorMacedonian : public TranslatorAdapter_1_6_0
         bool single)
     { // here s is one of " Module", " Struct" or " Union"
       // single is true implies a single file
-      QCString result=(QCString)"Документацијата за ";
+      QCString result="Документацијата за ";
       switch(compType)
       {
         case ClassDef::Class:      result+="оваа класа"; break;
@@ -1718,6 +1721,26 @@ class TranslatorMacedonian : public TranslatorAdapter_1_6_0
     {
       return "Ограничувања на Тип";
     }
+
+    virtual QCString trDayOfWeek(int dayOfWeek, bool, bool full)
+    {
+      static const char *days_short[]   = { "пон.", "вт.", "сре.", "чет.", "пет.", "саб.", "нед." };
+      static const char *days_full[]    = { "понеделник", "вторник", "среда", "четврток", "петок", "сабота", "недела" };
+      QCString text  = full? days_full[dayOfWeek-1] : days_short[dayOfWeek-1];
+      return text;
+    }
+    virtual QCString trMonth(int month, bool, bool full)
+    {
+      static const char *months_short[] = { "јан.", "фев.", "мар.", "апр.", "мај", "јун.", "јул.", "авг.", "септ.", "окт.", "ноем.", "дек." };
+      static const char *months_full[]  = { "јануари", "февруари", "март", "април", "мај", "јуни", "јули", "август", "септември", "октомври", "ноември", "декември" };
+      QCString text  = full? months_full[month-1] : months_short[month-1];
+      return text;
+    }
+    virtual QCString trDayPeriod(int period)
+    {
+      static const char *dayPeriod[] = { "претпл.", "попл." };
+      return dayPeriod[period];
+    }
 };
 
 #endif
